# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Communities UI views."""

from flask import Blueprint, current_app, render_template
from flask_babelex import lazy_gettext as _
from flask_login import current_user
from flask_menu import current_menu

#
# Error handlers
#
from invenio_app_rdm.users_ui.views.dashboard import communities, requests, \
    uploads

from ..searchapp import search_app_context


def not_found_error(error):
    """Handler for 'Not Found' errors."""
    return render_template(current_app.config['THEME_404_TEMPLATE']), 404


def record_permission_denied_error(error):
    """Handle permission denier error on record views."""
    if not current_user.is_authenticated:
        # trigger the flask-login unauthorized handler
        return current_app.login_manager.unauthorized()
    return render_template(current_app.config['THEME_403_TEMPLATE']), 403


#
# Registration
#
def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config.get("APP_RDM_USER_DASHBOARD_ROUTES")

    blueprint = Blueprint(
        "invenio_app_rdm_users",
        __name__,
        template_folder="../templates",
        static_folder='../static'
    )

    blueprint.add_url_rule(
        routes["uploads"],
        view_func=uploads,
    )

    # Settings tab routes
    blueprint.add_url_rule(
        routes["communities"],
        view_func=communities,
    )

    blueprint.add_url_rule(
        routes["requests"],
        view_func=requests,
    )

    @blueprint.before_app_first_request
    def register_menus():
        """Register community menu items."""
        user_dashboard = current_menu.submenu('dashboard')
        user_dashboard.submenu('uploads').register(
            'invenio_app_rdm_users.uploads',
            text=_('Uploads'),
            order=1,
        )
        user_dashboard.submenu('communities').register(
            'invenio_app_rdm_users.communities',
            text=_('Communities'),
            order=2,
        )
        user_dashboard.submenu('requests').register(
            'invenio_app_rdm_users.requests',
            text=_('Requests'),
            order=3,
        )

    # Register context processor
    blueprint.app_context_processor(search_app_context)

    return blueprint



from invenio_app_rdm.records_ui.views.filters import get_scheme_label


def test_get_scheme_label(app):
    # just test a couple of schemes
    assert 'PMID' == get_scheme_label('pmid')

    assert 'arXiv' == get_scheme_label('arxiv')

    assert 'Bibcode' == get_scheme_label('bibcode')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Request views module."""

from flask import g, render_template
from flask_login import current_user, login_required
from invenio_communities.members.services.request import CommunityInvitation
from invenio_communities.views.decorators import pass_community
from invenio_rdm_records.proxies import current_rdm_records_service
from invenio_rdm_records.requests import CommunitySubmission
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_records_resources.services.errors import PermissionDeniedError
from invenio_requests.resolvers.registry import ResolverRegistry
from invenio_requests.views.decorators import pass_request
from invenio_users_resources.proxies import current_user_resources
from sqlalchemy.orm.exc import NoResultFound


def _resolve_topic_draft(request):
    """Resolve the record in the topic when it is a draft."""
    permissions = {}

    recid = ResolverRegistry.resolve_entity_proxy(
        request["topic"]
    )._parse_ref_dict_id()
    try:
        record = current_rdm_records_service.read_draft(g.identity, recid)
        permissions.update(
            record.has_permissions_to(
                [
                    "edit",
                    "new_version",
                    "manage",
                    "update_draft",
                    "read_files",
                    "review",
                ]
            )
        )
        record_ui = UIJSONSerializer().serialize_object_to_dict(
            record.to_dict()
        )
        is_draft = True
    except NoResultFound:
        # record tab not displayed when the record is not found
        # the request is probably not open anymore
        record_ui = None
        is_draft = False

    return dict(
        permissions=permissions, record_ui=record_ui, is_draft=is_draft
    )


@login_required
@pass_request(expand=True)
def user_dashboard_request_view(request, **kwargs):
    """User dashboard request details view."""
    avatar = current_user_resources.users_service.links_item_tpl.expand(
        current_user
    )["avatar"]

    request_type = request["type"]

    is_draft_submission = request_type == CommunitySubmission.type_id
    is_invitation = request_type == CommunityInvitation.type_id
    if is_draft_submission:
        topic = _resolve_topic_draft(request)
        return render_template(
            "invenio_requests/community-submission/index.html",
            base_template="invenio_app_rdm/users/base.html",
            user_avatar=avatar,
            request=request.to_dict(),
            record=topic["record_ui"],
            permissions=topic["permissions"],
            is_preview=True,
            is_draft=topic["is_draft"],
            files=[],
        )

    elif is_invitation:
        return render_template(
            "invenio_requests/community-invitation/user_dashboard.html",
            base_template="invenio_app_rdm/users/base.html",
            user_avatar=avatar,
            request=request.to_dict(),
        )


@login_required
@pass_request(expand=True)
@pass_community
def community_dashboard_request_view(request, community, **kwargs):
    """Community dashboard requests details view."""
    request_type = request["type"]

    is_draft_submission = request_type == CommunitySubmission.type_id
    is_invitation = request_type == CommunityInvitation.type_id
    if is_draft_submission:
        permissions = community.has_permissions_to(
            ["update", "read", "search_requests", "search_invites"]
        )

        topic = _resolve_topic_draft(request)
        permissions.update(topic["permissions"])

        return render_template(
            "invenio_requests/community-submission/index.html",
            base_template="invenio_communities/details/base.html",
            request=request.to_dict(),
            record=topic["record_ui"],
            community=community.to_dict(),
            permissions=permissions,
            is_preview=True,
            is_draft=topic["is_draft"],
            files=[],
        )

    elif is_invitation:
        permissions = community.has_permissions_to(
            ["update", "read", "search_requests", "search_invites"]
        )
        if not permissions["can_search_invites"]:
            raise PermissionDeniedError()

        return render_template(
            "invenio_requests/community-invitation/community_dashboard.html",
            base_template="invenio_communities/details/members/base.html",
            request=request.to_dict(),
            community=community.to_dict(),
            permissions=permissions,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test that admin is not accessible even for admin users."""

from invenio_accounts.testutils import login_user_via_session


def test_admin_deny_access_admin(running_app, client, admin_user):
    login_user_via_session(client, email=admin_user.email)
    assert client.get('/admin/').status_code == 403
